@@ -43,16 +43,6 @@
 #  define ISSLASH(C) ((C) == DIRECTORY_SEPARATOR)
 # endif
 
-# ifndef FILE_SYSTEM_PREFIX_LEN
-#  define FILE_SYSTEM_PREFIX_LEN(Filename) 0
-# endif
-
-#ifndef SYMLINK_USES_UMASK
-# define UMASKED_SYMLINK(name1,name2,mode)    symlink(name1,name2)
-#else
-# define UMASKED_SYMLINK(name1,name2,mode)    umasked_symlink(name1,name2,mode)
-#endif				/* SYMLINK_USES_UMASK */
-
 /* Return 1 if an array of N objects, each of size S, cannot exist due
    to size arithmetic overflow.  S must be positive and N must be
    nonnegative.  This is a macro, not an inline function, so that it
@@ -92,38 +82,37 @@ int io_block_size = 512;
 
 struct new_cpio_header {
 	unsigned short c_magic;
-	unsigned long c_ino;
-	unsigned long c_mode;
-	unsigned long c_uid;
-	unsigned long c_gid;
-	unsigned long c_nlink;
-	unsigned long c_mtime;
-	unsigned long c_filesize;
-	long c_dev_maj;
-	long c_dev_min;
-	long c_rdev_maj;
-	long c_rdev_min;
-	unsigned long c_namesize;
-	unsigned long c_chksum;
+	union {
+		struct {
+			unsigned long c_ino;
+			unsigned long c_mode;
+			unsigned long c_uid;
+			unsigned long c_gid;
+			unsigned long c_nlink;
+			unsigned long c_mtime;
+			unsigned long c_filesize;
+			long c_dev_maj;
+			long c_dev_min;
+			long c_rdev_maj;
+			long c_rdev_min;
+			unsigned long c_namesize;
+			unsigned long c_chksum;
+		};
+		unsigned long c_hdr[13];
+	};
 	char *c_name;
 	char *c_tar_linkname;
 };
 
 /* Total number of bytes read and written for all files.  
-   Now that many tape drives hold more than 4Gb we need more than 32
-   bits to hold input_bytes and output_bytes.  But it's not worth
-   the trouble of adding special multi-precision arithmetic if the 
-   compiler doesn't support 64 bit ints since input_bytes and
-   output_bytes are only used to print the number of blocks copied.  */
-#ifdef __GNUC__
+ * Now that many tape drives hold more than 4Gb we need more than 32
+ *  bits to hold input_bytes and output_bytes.
+ */
 long long input_bytes, output_bytes;
-#else
-long input_bytes, output_bytes;
-#endif
 
 /* Allocate N bytes of memory dynamically, with error checking.  */
 
-void *xmalloc(size_t n)
+static void *xmalloc(size_t n)
 {
 	void *p;
 	if (xalloc_oversized(n, 1) || (!(p = malloc(n)) && n != 0)) {
@@ -134,22 +123,9 @@ void *xmalloc(size_t n)
 /*   return xnmalloc_inline (n, 1); */
 }
 
-/* Change the size of an allocated block of memory P to N bytes,
-   with error checking.  */
-
-void *xrealloc(void *p, size_t n)
-{
-	if (xalloc_oversized(n, 1) || (!(p = realloc(p, n)) && n != 0)) {
-		fprintf(stderr, "%s: memory exhausted\n", progname);
-		exit(1);
-	}
-	return p;
-/*   return xnrealloc_inline (p, n, 1); */
-}
-
 /* Clone STRING.  */
 
-char *xstrdup(char const *string)
+static char *xstrdup(char const *string)
 {
 	size_t s = strlen(string) + 1;
 	return memcpy(xmalloc(s), string, s);
@@ -187,7 +163,7 @@ static void tape_fill_input_buffer(int in_des, int num_bytes)
    must always be a multiple of 4 helps us (and our callers)
    insure this.  */
 
-void disk_empty_output_buffer(int out_des)
+static void disk_empty_output_buffer(int out_des)
 {
 	int bytes_written;
 
@@ -206,7 +182,7 @@ void disk_empty_output_buffer(int out_des)
 /* Copy NUM_BYTES of buffer IN_BUF to `out_buff', which may be partly full.
    When `out_buff' fills up, flush it to file descriptor OUT_DES.  */
 
-void disk_buffered_write(char *in_buf, int out_des, long num_bytes)
+static void disk_buffered_write(char *in_buf, int out_des, long num_bytes)
 {
 	register long bytes_left = num_bytes;	/* Bytes needing to be copied.  */
 	register long space_left;	/* Room left in output buffer.  */
@@ -235,7 +211,7 @@ void disk_buffered_write(char *in_buf, int out_des, long num_bytes)
    OUT_DES is the file descriptor for output;
    NUM_BYTES is the number of bytes to copy.  */
 
-void copy_files_tape_to_disk(int in_des, int out_des, long num_bytes)
+static void copy_files_tape_to_disk(int in_des, int out_des, long num_bytes)
 {
 	long size;
 
@@ -251,7 +227,7 @@ void copy_files_tape_to_disk(int in_des, int out_des, long num_bytes)
 }
 
 /* if IN_BUF is NULL, Skip the next NUM_BYTES bytes of file descriptor IN_DES. */
-void tape_buffered_read(char *in_buf, int in_des, long num_bytes)
+static void tape_buffered_read(char *in_buf, int in_des, long num_bytes)
 {
 	register long bytes_left = num_bytes;	/* Bytes needing to be copied.  */
 	register long space_left;	/* Bytes to copy from input buffer.  */
@@ -282,7 +258,7 @@ struct deferment {
 	struct new_cpio_header header;
 };
 
-struct deferment *create_deferment(struct new_cpio_header *file_hdr)
+static struct deferment *create_deferment(struct new_cpio_header *file_hdr)
 {
 	struct deferment *d;
 	d = (struct deferment *)xmalloc(sizeof(struct deferment));
@@ -292,13 +268,13 @@ struct deferment *create_deferment(struct new_cpio_header *file_hdr)
 	return d;
 }
 
-void free_deferment(struct deferment *d)
+static void free_deferment(struct deferment *d)
 {
 	free(d->header.c_name);
 	free(d);
 }
 
-int link_to_name(char *link_name, char *link_target)
+static int link_to_name(char *link_name, char *link_target)
 {
 	int res = link(link_target, link_name);
 	return res;
@@ -352,7 +328,7 @@ static void hash_insert(struct inode_val *new_value)
 
 /* Associate FILE_NAME with the inode NODE_NUM.  (Insert into hash table.)  */
 
-void
+static void
 add_inode(unsigned long node_num, char *file_name, unsigned long major_num,
 	  unsigned long minor_num)
 {
@@ -399,7 +375,7 @@ add_inode(unsigned long node_num, char *file_name, unsigned long major_num,
 	hash_num++;
 }
 
-char *find_inode_file(unsigned long node_num, unsigned long major_num,
+static char *find_inode_file(unsigned long node_num, unsigned long major_num,
 		      unsigned long minor_num)
 {
 	int start;		/* Initial hash location.  */
@@ -439,7 +415,7 @@ char *find_inode_file(unsigned long node_num, unsigned long major_num,
    numbers is found, try and create another link to it using
    link_to_name, and return 0 for success and -1 for failure.  */
 
-int
+static int
 link_to_maj_min_ino(char *file_name, int st_dev_maj, int st_dev_min, int st_ino)
 {
 	int link_res;
@@ -457,7 +433,7 @@ link_to_maj_min_ino(char *file_name, int st_dev_maj, int st_dev_min, int st_ino)
 static void copyin_regular_file(struct new_cpio_header *file_hdr,
 				int in_file_des);
 
-void warn_junk_bytes(long bytes_skipped)
+static void warn_junk_bytes(long bytes_skipped)
 {
 	fprintf(stderr, "%s: warning: skipped %ld byte(s) of junk\n",
 		progname, bytes_skipped);
@@ -578,7 +554,7 @@ static void create_defered_links(struct new_cpio_header *file_hdr)
    we are done reading the whole archive.  Write out all of these
    empty links that are still on the deferments list.  */
 
-static void create_final_defers()
+static void create_final_defers(void)
 {
 	struct deferment *d;
 	int link_res;
@@ -720,9 +696,9 @@ copyin_regular_file(struct new_cpio_header *file_hdr, int in_file_des)
    NAME has no file name components because it is all slashes, return
    NAME if it is empty, the address of its last slash otherwise.  */
 
-char *base_name(char const *name)
+static char *base_name(char const *name)
 {
-	char const *base = name + FILE_SYSTEM_PREFIX_LEN(name);
+	char const *base = name;
 	char const *p;
 
 	for (p = base; *p; p++) {
@@ -752,7 +728,7 @@ char *base_name(char const *name)
    value returned by base_name.  Act like strlen (NAME), except omit
    redundant trailing slashes.  */
 
-size_t base_len(char const *name)
+static size_t base_len(char const *name)
 {
 	size_t len;
 
@@ -769,7 +745,7 @@ size_t base_len(char const *name)
    the Unix rename and rmdir system calls return an "Invalid argument" error
    when given a path that ends in "/" (except for the root directory).  */
 
-bool strip_trailing_slashes(char *path)
+static bool strip_trailing_slashes(char *path)
 {
 	char *base = base_name(path);
 	char *base_lim = base + base_len(base);
@@ -869,9 +845,9 @@ static void copyin_link(struct new_cpio_header *file_hdr, int in_file_des)
 	tape_buffered_read(link_name, in_file_des, file_hdr->c_filesize);
 	tape_skip_padding(in_file_des, file_hdr->c_filesize);
 
-	res = UMASKED_SYMLINK(link_name, file_hdr->c_name, file_hdr->c_mode);
+	res = symlink(link_name, file_hdr->c_name);
 	if (res < 0) {
-		fprintf(stderr, "%s: UMASKED_SYMLINK %s: %s\n",
+		fprintf(stderr, "%s: symlink %s: %s\n",
 			progname, file_hdr->c_name, strerror(errno));
 		free(link_name);
 		return;
@@ -903,20 +879,14 @@ static void copyin_file(struct new_cpio_header *file_hdr, int in_file_des)
 
 	case S_IFCHR:
 	case S_IFBLK:
-#ifdef S_IFSOCK
 	case S_IFSOCK:
-#endif
-#ifdef S_IFIFO
 	case S_IFIFO:
-#endif
 		copyin_device(file_hdr);
 		break;
 
-#ifdef S_IFLNK
 	case S_IFLNK:
 		copyin_link(file_hdr, in_file_des);
 		break;
-#endif
 
 	default:
 		fprintf(stderr, "%s: %s: unknown file type\n",
@@ -930,20 +900,19 @@ static void copyin_file(struct new_cpio_header *file_hdr, int in_file_des)
    file descriptor IN_DES, except for the magic number, which is
    already filled in.  */
 
-void read_in_new_ascii(struct new_cpio_header *file_hdr, int in_des)
+static void read_in_new_ascii(struct new_cpio_header *file_hdr, int in_des)
 {
-	char ascii_header[112];
+	char ascii_header[13*8], *ah, hexbuf[9];
+	int i;
 
-	tape_buffered_read(ascii_header, in_des, 104L);
-	ascii_header[104] = '\0';
-	sscanf(ascii_header,
-	       "%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx",
-	       &file_hdr->c_ino, &file_hdr->c_mode, &file_hdr->c_uid,
-	       &file_hdr->c_gid, &file_hdr->c_nlink, &file_hdr->c_mtime,
-	       &file_hdr->c_filesize, &file_hdr->c_dev_maj,
-	       &file_hdr->c_dev_min, &file_hdr->c_rdev_maj,
-	       &file_hdr->c_rdev_min, &file_hdr->c_namesize,
-	       &file_hdr->c_chksum);
+	tape_buffered_read(ascii_header, in_des, 13*8);
+	ah = ascii_header;
+	hexbuf[8] = '\0';
+	for (i = 0; i < 13; i++) {
+		memcpy(hexbuf, ah, 8);
+		file_hdr->c_hdr[i] = strtoul(hexbuf, NULL, 16);
+		ah += 8;
+	}
 	/* Read file name from input.  */
 	if (file_hdr->c_name != NULL)
 		free(file_hdr->c_name);
@@ -963,7 +932,7 @@ void read_in_new_ascii(struct new_cpio_header *file_hdr, int in_des)
 /* Read the header, including the name of the file, from file
    descriptor IN_DES into FILE_HDR.  */
 
-void read_in_header(struct new_cpio_header *file_hdr, int in_des)
+static void read_in_header(struct new_cpio_header *file_hdr, int in_des)
 {
 	long bytes_skipped = 0;	/* Bytes of junk found before magic number.  */
 
@@ -991,7 +960,7 @@ void read_in_header(struct new_cpio_header *file_hdr, int in_des)
 /* Read the collection from standard input and create files
    in the file system.  */
 
-void process_copy_in()
+static void process_copy_in(void)
 {
 	char done = false;	/* True if trailer reached.  */
 	struct new_cpio_header file_hdr;	/* Output header information.  */
@@ -1033,7 +1002,7 @@ void process_copy_in()
    initialize all variables associated with the input and output
    buffers.  */
 
-void initialize_buffers()
+static void initialize_buffers(void)
 {
 	int in_buf_size, out_buf_size;
 
